#! /usr/bin/env node
/* eslint-disable no-await-in-loop, import/no-nodejs-modules */
/* eslint-env node */
// @ts-check
/**
 * @author <ziad.khoury.hanna@oceandatalab.com>
 *
 * Copyright (C) 2014-2024 OceanDataLab
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
import {cwd} from 'node:process';
import {fileURLToPath, pathToFileURL} from 'node:url';
import {basename, relative} from 'node:path';
import fs from 'node:fs/promises';

/**
 * @typedef TOCEntry
 * @prop {number} level
 * @prop {string} html
 * @prop {string} text
 * @prop {string} href
 * @prop {Array<TOCEntry>} children
 */
/** @typedef {Array<TOCEntry>} TOC */
/**
 * @typedef MDFile
 * @prop {string | URL} filePath
 * @prop {Record<string, string>} frontmatter
 * @prop {string} title
 * @prop {string} body
 * @prop {TOC} toc
 */
/**
 * @typedef HTMLHeadingReGroups
 * @prop {string} level
 * @prop {string} html
 * @prop {string} text
 * @prop {string} href
 */
/**
 * @typedef HTMLLinkReGroups
 * @prop {string} html
 * @prop {string} href
 */

const TITLE = 'Syntool Web Documentation';
const ROOT_DIR = new URL('../', import.meta.url);
const DOCS_DIR = new URL('doc-src/', ROOT_DIR);
const DIST_DIR = new URL('doc/', ROOT_DIR);
const TMPL_PATH = new URL('_tmpl.html', DOCS_DIR);
const LINK_ICON = '<svg class="octicon octicon-link" viewBox="0 0 16 16" version="1.1" width="16" height="16" aria-hidden="true"><path fill-rule="evenodd" d="M7.775 3.275a.75.75 0 001.06 1.06l1.25-1.25a2 2 0 112.83 2.83l-2.5 2.5a2 2 0 01-2.83 0 .75.75 0 00-1.06 1.06 3.5 3.5 0 004.95 0l2.5-2.5a3.5 3.5 0 00-4.95-4.95l-1.25 1.25zm-4.69 9.64a2 2 0 010-2.83l2.5-2.5a2 2 0 012.83 0 .75.75 0 001.06-1.06 3.5 3.5 0 00-4.95 0l-2.5 2.5a3.5 3.5 0 004.95 4.95l1.25-1.25a.75.75 0 00-1.06-1.06l-1.25 1.25a2 2 0 01-2.83 0z"></path></svg>';
const HTML_HEADING_RE = /<h(?<level>\d) class="heading-element">\s*(?<html>.*?)\s*<\/h\k<level>><a id="[^"]*" class="anchor" aria-label="Permalink: (?<text>[^"]*)" href="(?<href>#[^"]*)">/g;
const HTML_LINK_RE = /<a href="(?<href>[^"]*)">(?<html>.*?)<\/a>/g;

function ttitle(...args) {
  console.log('::', ...args);
}
function theader(...args) {
  console.log('##', ...args);
}
function techo(...args) {
  console.log('-> #', ...args);
}
function tcmd(...args) {
  console.log('->', ...args);
}

/**
 * @param {string | URL} path
 * @returns {Promise<void>}
 */
async function mkdir(path) {
  tcmd('mkdir -p', relpath(path));
  await fs.mkdir(path, {recursive: true});
}
/**
 * @param {string | URL} source
 * @param {string | URL} destination
 * @returns {Promise<void>}
 */
function cp(source, destination) {
  tcmd('cp -r', relpath(source), relpath(destination));
  return fs.cp(source, destination, {
    dereference: true,
    mode: fs.constants.COPYFILE_FICLONE,
    recursive: true,
  });
}
/**
 * @param {string | URL} p
 * @param {string | URL} [from]
 * @returns {string}
 */
function relpath(p, from = cwd()) {
  if (!URL.canParse(from)) {
    from = pathToFileURL(/** @type {any} */ (from));
  }
  if (!URL.canParse(p)) {
    p = pathToFileURL(/** @type {any} */ (p));
  }
  return relative(fileURLToPath(from), fileURLToPath(p));
}

/**
 * @param {string | URL} mdFilePath
 * @returns {Promise<Record<string, string>>}
 */
async function getFrontmatter(mdFilePath) {
  /** @type {Record<string, string>} */
  const frontmatter = {};
  let inFrontmatter = false;
  const file = await fs.open(mdFilePath);
  for await (const line of file.readLines({encoding: 'utf8', start: 0})) {
    if (inFrontmatter) {
      const match = line.match(/^\s*([a-z0-9_-]+)\s*:(.*)$/i);
      if (match) {
        frontmatter[match[1]] = match[2].trim();
      } else if (line === '-->') {
        break;
      }
    } else if (line === '<!--') {
      inFrontmatter = true;
    } else if (/^\s*#/.test(line)) {
      break;
    }
  }
  return frontmatter;
}

/**
 * @param {string | URL} mdFilePath
 * @param {AbortSignal} [signal]
 * @returns {Promise<string>}
 */
async function compileMDFile(mdFilePath, signal) {
  if (!URL.canParse(mdFilePath)) {
    mdFilePath = pathToFileURL(/** @type {any} */ (mdFilePath));
  }
  const mdFileName = basename(fileURLToPath(mdFilePath), '.md');
  const htmlFilePath = new URL('.' + mdFileName + '.html', mdFilePath);

  try {
    const {mtime: htmlMTime} = await fs.stat(htmlFilePath);
    const {mtime: mdMTime} = await fs.stat(mdFilePath);
    if (htmlMTime > mdMTime) {
      return fs.readFile(htmlFilePath, {encoding: 'utf8'});
    }
  } catch (error) {
    if (error.code !== 'ENOENT') {
      throw error;
    }
  }

  const mdFile = await fs.open(mdFilePath);
  const response = await fetch('https://api.github.com/markdown/raw', {
    method: 'POST',
    headers: {
      'Content-Type': 'text/plain',
      Accept: 'application/vnd.github.v3+json',
    },
    // @ts-expect-error
    body: mdFile.createReadStream(),
    duplex: 'half',
    signal,
  });
  await mdFile.close();
  if (!response.ok) {
    throw new Error(`HTTP error! Status: ${response.status}`);
  }
  const html = await response.text();
  try {
    await fs.writeFile(htmlFilePath, html, {encoding: 'utf8', signal});
  } catch (error) {
    console.error(error);
  }
  return html;
}

/**
 * @param {string | URL} mdFilePath
 * @param {Set<string>} headingHrefs
 * @param {AbortSignal} [signal]
 * @returns {Promise<MDFile>}
 */
async function prepareFile(mdFilePath, headingHrefs, signal) {
  techo(relpath(mdFilePath));

  const [frontmatter, body] = await Promise.all([
    getFrontmatter(mdFilePath),
    compileMDFile(mdFilePath, signal),
  ]);
  const fileHref = relpath(mdFilePath, DOCS_DIR).replace(/.md$/g, '.html');
  /** @type {TOC} */
  const toc = [];
  /** @type {TOC} */
  const tocStack = [];
  /** @type {TOCEntry} */
  let currentToc = {level: 0, html: '', text: '', href: '', children: toc};
  headingHrefs.add(fileHref);
  for (
    let {groups: {level, html, text, href}}
    of /** @type {IterableIterator<{groups: HTMLHeadingReGroups}>} */ (
      /** @type {unknown} */ (body.matchAll(HTML_HEADING_RE))
    )
  ) {
    html = html.replace(/<a\s+.*?>(.*?)<\/a>/g, '$1');
    href = fileHref + href.replace(/.md(\?|#|$)/g, '.html$1');

    headingHrefs.add(href);
    const l = parseInt(level, 10);
    if (l > 3) {
      continue;
    }

    while (currentToc.level >= l) {
      currentToc = /** @type {TOCEntry} */ (tocStack.pop());
    }

    /** @type {TOCEntry} */
    const child = {level: l, html, text, href, children: []};
    currentToc.children.push(child);
    tocStack.push(currentToc);
    currentToc = child;
  }
  const title = frontmatter.title || toc[0].text;
  return {filePath: mdFilePath, frontmatter, title, body, toc};
}

/**
 * @param {Array<TOCEntry>} children
 * @param {number} [maxLevel]
 * @param {TOC} [currentToc]
 * @returns {string}
 */
function renderTOCChildren(children, maxLevel = Infinity, currentToc = []) {
  const currentHrefs = new Set();
  for (const {href} of currentToc) {
    currentHrefs.add(href);
  }
  const items = [];
  for (const {level, html, href, children: c} of children) {
    if (level <= maxLevel) {
      const isCurrent = currentHrefs.has(href);
      const heading = (
        isCurrent
        ? html
        : `<a href="${href}">${html}</a>`
      );
      const sub = renderTOCChildren(c, maxLevel);
      const detailsAttrs = isCurrent ? ' open' : '';
      const contents = (
        sub
        ? `<details${detailsAttrs}><summary>${heading}</summary>${sub}`
        : heading
      );
      const liClasses = [isCurrent && 'current', sub && 'no-marker'].filter(Boolean);
      const liAttrs = (
        liClasses.length > 0
        ? ` class="${liClasses.join(' ')}"`
        : ''
      );
      items.push(`<li${liAttrs}>${contents}</li>`);
    }
  }
  return items.length > 1 ? `<ul>${items.join('\n')}</ul>` : '';
}

/**
 * @param {MDFile} md
 * @param {TOC} toc
 * @param {AbortSignal} [signal]
 * @returns {Promise<void>}
 */
async function compileFile(md, toc, signal) {
  const {filePath, title, body, toc: fileToc} = md;
  const htmlFilePath = new URL(
    relpath(filePath, DOCS_DIR).replace(/\.md$/, '.html'),
    DIST_DIR
  );

  techo(`${relpath(htmlFilePath)} [${title}]`);

  const pageTitle = (
    !title || title === TITLE
    ? TITLE
    : `${title} • ${TITLE}`
  );
  const htmlToc = (
    '<nav class="toc-wrapper" aria-labelledby="toc">'
    + '<h2 id="toc"">Table of Contents</h2>'
    + renderTOCChildren(toc, 3, fileToc)
    + '</nav>'
  );
  const html = (
    (await fs.readFile(TMPL_PATH, {encoding: 'utf8', signal}))
    .replace(/<!-- {{T__TITLE}} -->/g, pageTitle)
    .replace(/<!-- {{T__TOC}} -->/g, `<div class="markdown-toc">${htmlToc}</div>`)
    .replace(/<!-- {{T__BODY}} -->/g, `<main class="markdown-body">${body}</main>`)
    .replace(/ id="user-content-/g, ' id="')
    .replace(/<a href="(.*).md(#.*)?">/g, '<a href="$1.html$2">')
    .replace(/<span aria-hidden="true" class="octicon octicon-link"><\/span>/g, LINK_ICON)
  );

  await fs.writeFile(htmlFilePath, html, {encoding: 'utf8', signal});
}

/**
 * @param {TOC} toc
 * @param {AbortSignal} [signal]
 * @returns {Promise<void>}
 */
async function compileIndex(toc, signal) {
  const htmlFilePath = new URL('index.html', DIST_DIR);

  techo(`${relpath(htmlFilePath)}`);

  const body = (
    `<div class="markdown-heading"><h1 class="heading-element">${TITLE}</h1></div>`
    + renderTOCChildren(toc, 2)
  );

  const html = (
    (await fs.readFile(TMPL_PATH, {encoding: 'utf8', signal}))
    .replace(/<!-- {{T__TITLE}} -->/g, TITLE)
    .replace(/<!-- {{T__TOC}} -->/g, '')
    .replace(/<!-- {{T__BODY}} -->/g, `<main class="markdown-body">${body}</main>`)
    .replace(/<a href="(.*).md(#.*)?">/g, '<a href="$1.html$2">')
  );

  await fs.writeFile(htmlFilePath, html, {encoding: 'utf8', signal});
}

/**
 * @param {MDFile} md
 * @param {Readonly<Set<string>>} headingHrefs
 * @returns {boolean}
 */
function checkLinks(md, headingHrefs) {
  const {filePath, title, body} = md;
  const fileHref = relpath(filePath, DOCS_DIR).replace(/.md$/g, '.html');
  let hasDeadLink = false;

  techo(`${relpath(filePath, DOCS_DIR)} [${title}]`);
  for (
    const {groups: {html, href}}
    of /** @type {IterableIterator<{groups: HTMLLinkReGroups}>} */ (
      /** @type {unknown} */ (body.matchAll(HTML_LINK_RE))
    )
  ) {
    if (URL.canParse(href)) {
      continue;
    }
    const htmlHref = (
      (href[0] === '#' ? fileHref + href : href)
      .replace(/.md(\?|#|$)/g, '.html$1')
    );
    if (!headingHrefs.has(htmlHref)) {
      console.error(`[${html}]: ${href}`);
      hasDeadLink = true;
    }
  }
  return !hasDeadLink;
}

/**
 * @param {AbortSignal} [signal]
 */
async function build(signal) {
  ttitle('Build documentation');
  try {
    theader('Collect documentation files');
    /** @type {Set<string | URL>} */
    const mdFiles = new Set();
    /** @type {Set<string | URL>} */
    const utilFiles = new Set();
    /** @type {Set<string | URL>} */
    const staticResources = new Set();
    for await (const dirent of await fs.opendir(DOCS_DIR)) {
      signal?.throwIfAborted();
      if (dirent.isFile() || dirent.isSymbolicLink()) {
        if (dirent.name.startsWith('.')) {
          const mdFileName = dirent.name.slice(1).replace(/\.html/, '.md');
          try {
            await fs.stat(new URL(mdFileName, DOCS_DIR));
          } catch (error) {
            await fs.rm(new URL(dirent.name, DOCS_DIR));
          }
        } else {
          (
            dirent.name.startsWith('_') ? utilFiles :
            dirent.name.endsWith('.md') ? mdFiles :
            staticResources
          ).add(fileURLToPath(new URL(dirent.name, DOCS_DIR)));
        }
      }
    }

    signal?.throwIfAborted();
    theader('Prepare markdown files');
    /** @type {Array<MDFile>} */
    const mds = [];
    /** @type {TOC} */
    const toc = [];
    /** @type {Set<string>} */
    const headingHrefs = new Set();
    for (const mdFile of [...mdFiles].sort()) {
      signal?.throwIfAborted();
      const md = await prepareFile(mdFile, headingHrefs, signal);
      mds.push(md);
      toc.push(...md.toc);
    }

    signal?.throwIfAborted();
    theader('Check for dead-links');
    for (const md of mds) {
      checkLinks(md, headingHrefs);
    }

    signal?.throwIfAborted();
    theader('Create output directory');
    await mkdir(DIST_DIR);

    signal?.throwIfAborted();
    theader('Build markdown files');
    await compileIndex(toc, signal);
    for (const md of mds) {
      signal?.throwIfAborted();
      await compileFile(md, toc, signal);
    }

    signal?.throwIfAborted();
    theader('Copy static resources');
    for (const sourceFile of staticResources) {
      signal?.throwIfAborted();
      await cp(sourceFile, new URL(relpath(sourceFile, DOCS_DIR), DIST_DIR));
    }

    theader('Done');
  } catch (error) {
    if (error.name === 'AbortError') {
      theader('Aborted');
    } else {
      console.error(error);
      theader('Failed');
    }
  }
}

await build();

/**
 * @param {number} ms
 * @param {(this: void) => void} fn
 * @returns {(this: void) => void}
 */
function debounce(ms, fn) {
  let timeout = null;
  function invoke() {
    timeout = null;
    fn();
  }
  return function() {
    if (timeout === null) {
      timeout = setTimeout(invoke, ms);
    }
  };
}

if (process.argv[2] === 'watch') {
  ttitle('Watching');

  let abortController = new AbortController();
  const rebuild = debounce(2_000, () => {
    abortController.abort();
    abortController = new AbortController();
    build(abortController.signal);
  });

  for await (const event of fs.watch(DOCS_DIR)) {
    if (!event.filename?.startsWith('.')) {
      rebuild();
    }
  }
}
